#ifndef ECHIQUIER_HPP
#define ECHIQUIER_HPP

#include "square.hpp"
#include "pieces/piece.hpp"
#include "pieces/pion.hpp"
#include <list>
#include <vector>
#include "move.hpp"

class Piece;

/// @brief Représente l'échiquier du jeu d'échec
class Echiquier{
	private:
		/// @brief Cases du jeu d'échec
		Piece* board[8][8]{};
		/// @brief Toutes les pièces présentes dans le jeu
		std::vector<Piece*> pieces[2];
		/// @brief Listes de tous les coups précédents
		std::list<Move> moves {};

		/// @brief Indique si un échec a lieu
		bool is_check = false;

	public:
		Echiquier();
		~Echiquier();

		/// @brief Place une pièce à un endroit sur le plateau
		/// @param piece
		/// @warning Attention : pas de retrait de la position précédente !
		void set_pos(Piece* piece, Square pos);

		/// @brief Retourne la position canonique de l'échiquier
		/// @return Position canonique
		std::string canonical_position() const;

		/// @brief Affiche l'échiquier
		void print_board() const;

		/// @brief Permet d'effectuer un mouvement de pièce
		/// @param from Pièce à déplacer
		/// @param to Destination
		/// @return true si le mouvement a été effectué, false sinon
		bool move(Square from, Square to, Color player);

		/// @brief Place les pions et pièces à leur emplacement de départ.
		void setup();

		/// @brief Récupère la liste des coups effectués lors de la partie
		/// @return Liste des coups
		const std::list<Move>& get_moves() const;

		/// @brief Récupère une pièce se trouvant à la position donnée
		/// @param pos Position de la pièce
		/// @return Pointeur vers la pièce si elle a été trouvée, nullptr sinon.
		Piece* get_piece(Square pos) const;

		/// @brief Récupère une pièce d'un joueur à partir de son id.
		/// @param id id de la pièce
		/// @param player Joueur a qui appartient la pièce
		/// @return Pointeur vers la pièce si elle a été trouvée, nullptr sinon.
		Piece* get_piece_from_id(int id, Color player) const;

		/// @brief Vérifie si une case est menacée par un joueur
		/// @param player Joueur pour qui la case peut être une menace
		/// @param pos Position de la case
		/// @param exclude Pièce à exclure dans la détection de la menace
		/// @return true si la case est une menace pour le joueur, false sinon
		bool check_check(Color player, Square pos, Piece* exclude = nullptr) const;

		/// @brief Définit s'il y a un échec ou non
		/// @param is_check true s'il y a un échec, false sinon
		void set_is_check(bool is_check) { this->is_check = is_check; }

		/// @brief Vérifie s'il y a un échec
		/// @return true s'il y a un échec, false sinon
		bool get_is_check() const { return is_check; }

		/// @brief Vérifie si le coup entre from et to est légal pour le joueur player
		/// @param from Case d'origine
		/// @param to Case de destination
		/// @param player Joueur à l'origine du coup
		/// @return true si origine, false sinon
		bool check_move(Square from, Square to, Color player);

		/// @brief Vérifie si le joueur est en situation d'échec et mat
		/// @param player Joueur
		/// @return true si échec et mat, false sinon.
		bool check_check_mate(Color player);

		/// @brief Vérifie si le joueur est en situation de pat
		/// @param player Joueur
		/// @return true si échec et mat, false sinon.
		bool check_stalemate(Color player);

		/// @brief Vérifie si un joueur peut roquer ou non
		/// @param player Joueur voulant roquer
		/// @param big true si c'est un grand roque, false sinon
		/// @return true si le roque est possible, false sinon
		bool check_castling(Color player, bool big) const;

		/// @brief Effectue le roque d'un joueur
		/// @param player Joueur voulant roquer
		/// @param big true si c'est un grand roque, false sinon
		/// @return true si le roque a eu lieu, false sinon
		bool castling(Color player, bool big);

		/// @brief Permet de promouvoir un pion en une autre pièce
		/// @param pion Pion à promouvoir
		/// @param type Type de la pièce dont le pion sera promu. ("Q" pour une reine, "R" pour une tour, "B" pour un fou, "K" pour un cavalier)
		/// @return true si la promotion a eu lieu, false sinon
		bool promote(Pion *pion, const std::string& type);
};

#endif