#ifndef PIECES_PIECE_HPP
#define PIECES_PIECE_HPP

#include <string>
#include <iostream>
#include "enums.hpp"
#include "square.hpp"

class Echiquier;

/// @brief Classe décrivant une pièce
class Piece {
	private:
		/// @brief Nom de la pièce
		std::string name;

		/// @brief Nom de la pièce canonique
		std::string canon_name;

		/// @brief Couleur de la pièce
		Color color;
		
		/// @brief ID de la pièce
		/// @warning Il est préférable que l'ID soit unique
		int id;

		/// @brief Position de la pièce
		Square pos;

		/// @brief true si la pièce a bougé durant la partie, false sinon
		bool has_moved{false};

		/// @brief Définit si la pièce a été mangée ou non
		bool is_active{true};

	protected:
		/// @brief Echiquier possédant la pièce
		Echiquier* echiquier;

	public:
		/// @brief Vérifie si un mouvement de la case origine à la case destination est valide
		/// @param origine Case d'origine
		/// @param destination Destination
		/// @return Vrai si le mouvement est légal, faux sinon
		virtual bool est_mouvement_legal(Square origine, Square destination) const;

		/// @brief Vérifie si la pièce a été mangée ou non
		/// @return true si la pièce n'est pas mangée, false sinon
		bool get_is_active() const {
			return is_active;
		}

		/// @brief Définit si la pièce est mangée ou non
		/// @param is_active true si la pièce n'est pas mangée, false sinon
		void set_is_active(bool is_active) {
			this->is_active = is_active;
		}

		/// @brief Vérifie si la pièce est une menace pour l'adversaire
		/// @param origine Position d'origine
		/// @param destination Position de destination
		/// @return true si la pièce est une menace, false sinon
		virtual bool est_menace(Square origine, Square destination) const;

		/// @return Nom de la pièce a affiché sur l'échiquier
		virtual std::string get_name() const { return name; }

		/// @brief Fonction appelée quand une pièce bouge
		/// @param from Case d'origine
		virtual void move_event(Square from) {
			has_moved = true;
		}

		/// @return Couleur de la pièce
		Color get_color() const { return color; }

		/// @return Retourne l'ID de la pièce
		int get_id() const { return id; }

		/// @brief Constructeur de la pièce
		/// @param name Nom de la pièce
		/// @param color Couleur de la pièce
		/// @param id ID de la pièce
		/// @param start_pos Position de départ lors de l'initialisation du jeu
		/// @param echiquier Echiquier où se trouve la pièce
		Piece(std::string name, std::string canon_name, Color color, int id, Square start_pos, Echiquier* echiquier);

		/// Renvoie son écriture canonique (un fou en b7 donnera ("Bb7"))
		/// @param show_color Affiche la couleur de la pièce ou non
		/// @param show_letter Affiche la lettre de la pièce ou non
		/// @return écriture canonique de la pièce
		std::string canonical(bool show_color = true, bool show_letter = true) const;

		/// @brief Destructeur virtuel
		virtual ~Piece();

		/// @brief Vérifie si la pièce a déjà bougée ou non
		/// @return true si la pièce a bougé, faux sinon
		bool get_has_moved() const { return has_moved; }

		/// @brief Définit si la pièce a déjà bougée ou non
		/// @param has_moved true si la pièce a bougé, faux sinon
		void set_has_moved(bool has_moved) { this->has_moved = has_moved; }

		/// @brief Récupère la position actuelle de la pièce
		/// @return position de la pièce
		Square get_pos() const { return pos; }

		/// @brief Définit la position de la pièce
		/// @param pos Position de la pièce
		void set_pos(const Square pos) { this->pos = pos; }

		/// @brief Vérifie si une pièce est égale à une autre
		/// @param other Pièce à comparer
		/// @return vrai si la pièce est égale à other, false sinon
		bool operator== (const Piece &other) const;
};


#endif
